@@ -20,11 +20,13 @@ from saml2.sigver import REQ_ORDER
 from saml2.sigver import RESP_ORDER
 from saml2.sigver import SIGNER_ALGS
 import six
+from saml2.xmldsig import SIG_ALLOWED_ALG
 
 logger = logging.getLogger(__name__)
 
 try:
     from xml.etree import cElementTree as ElementTree
+
     if ElementTree.VERSION < '1.3.0':
         # cElementTree has no support for register_namespace
         # neither _namespace_map, thus we sacrify performance
@@ -35,14 +37,38 @@ except ImportError:
         import cElementTree as ElementTree
     except ImportError:
         from elementtree import ElementTree
+import defusedxml.ElementTree
 
 NAMESPACE = "http://schemas.xmlsoap.org/soap/envelope/"
-FORM_SPEC = """<form method="post" action="%s">
-   <input type="hidden" name="%s" value="%s" />
-   <input type="hidden" name="RelayState" value="%s" />
-   <input type="submit" value="Submit" />
-</form>"""
 
+FORM_SPEC = """\
+<!DOCTYPE html>
+<html>
+    <head>
+        <meta charset="utf-8" />
+    </head>
+    <body onload="document.forms[0].submit()">
+        <noscript>
+            <p>
+                <strong>Note:</strong> Since your browser does not support JavaScript,
+                you must press the Continue button once to proceed.
+            </p>
+        </noscript>
+        
+        <form action="{action}" method="post">
+            <div>
+                <input type="hidden" name="RelayState" value="{relay_state}"/>                
+                                
+                <input type="hidden" name="{saml_type}" value="{saml_response}"/>
+            </div>
+            <noscript>
+                <div>
+                    <input type="submit" value="Continue"/>
+                </div>
+            </noscript>
+        </form>
+            </body>
+</html>"""
 
 def http_form_post_message(message, location, relay_state="",
                            typ="SAMLRequest", **kwargs):
@@ -55,8 +81,6 @@ def http_form_post_message(message, location, relay_state="",
     :param relay_state: for preserving and conveying state information
     :return: A tuple containing header information and a HTML message.
     """
-    response = ["<head>", """<title>SAML 2.0 POST</title>""", "</head><body>"]
-
     if not isinstance(message, six.string_types):
         message = str(message)
     if not isinstance(message, six.binary_type):
@@ -68,19 +92,45 @@ def http_form_post_message(message, location, relay_state="",
         _msg = message
     _msg = _msg.decode('ascii')
 
-    response.append(FORM_SPEC % (location, typ, _msg, relay_state))
+    args = {
+       'action'        : location,
+       'saml_type'     : typ,
+       'relay_state'   : relay_state,
+       'saml_response' : _msg
+       }
 
-    response.append("""<script type="text/javascript">""")
-    response.append("     window.onload = function ()")
-    response.append(" { document.forms[0].submit(); }")
-    response.append("""</script>""")
-    response.append("</body>")
+    response = FORM_SPEC.format(**args)
 
     return {"headers": [("Content-type", "text/html")], "data": response}
 
+def http_post_message(message, relay_state="", typ="SAMLRequest", **kwargs):
+    """
+
+    :param message: The message
+    :param relay_state: for preserving and conveying state information
+    :return: A tuple containing header information and a HTML message.
+    """
+    if not isinstance(message, six.string_types):
+        message = str(message)
+    if not isinstance(message, six.binary_type):
+        message = message.encode('utf-8')
+
+    if typ == "SAMLRequest" or typ == "SAMLResponse":
+        _msg = base64.b64encode(message)
+    else:
+        _msg = message
+    _msg = _msg.decode('ascii')
+
+    part = {typ: _msg}
+    if relay_state:
+        part["RelayState"] = relay_state
+
+    return {"headers": [("Content-type", 'application/x-www-form-urlencoded')],
+            "data": urlencode(part)}
+
 
 def http_redirect_message(message, location, relay_state="", typ="SAMLRequest",
-                          sigalg=None, key=None, **kwargs):
+                          sigalg='', signer=None, **kwargs):
     """The HTTP Redirect binding defines a mechanism by which SAML protocol
     messages can be transmitted within URL parameters.
     Messages are encoded for use with this binding using a URL encoding
@@ -92,8 +142,9 @@ def http_redirect_message(message, location, relay_state="", typ="SAMLRequest",
     :param location: Where the message should be posted to
     :param relay_state: for preserving and conveying state information
     :param typ: What type of message it is SAMLRequest/SAMLResponse/SAMLart
-    :param sigalg: The signature algorithm to use.
-    :param key: Key to use for signing
+    :param sigalg: Which algorithm the signature function will use to sign
+        the message
+    :param signer: A signature function that can be used to sign the message
     :return: A tuple containing header information and a HTML message.
     """
 
@@ -115,20 +166,15 @@ def http_redirect_message(message, location, relay_state="", typ="SAMLRequest",
     if relay_state:
         args["RelayState"] = relay_state
 
-    if sigalg:
-        # sigalgs, one of the ones defined in xmldsig
-
+    if signer:
+        # sigalgs, should be one defined in xmldsig
+        assert sigalg in [b for a, b in SIG_ALLOWED_ALG]
         args["SigAlg"] = sigalg
 
-        try:
-            signer = SIGNER_ALGS[sigalg]
-        except:
-            raise Unsupported("Signing algorithm")
-        else:
-            string = "&".join([urlencode({k: args[k]})
-                               for k in _order if k in args]).encode('ascii')
-            args["Signature"] = base64.b64encode(signer.sign(string, key))
-            string = urlencode(args)
+        string = "&".join([urlencode({k: args[k]})
+                           for k in _order if k in args]).encode('ascii')
+        args["Signature"] = base64.b64encode(signer.sign(string))
+        string = urlencode(args)
     else:
         string = urlencode(args)
 
@@ -169,18 +215,18 @@ def make_soap_enveloped_saml_thingy(thingy, header_parts=None):
     if isinstance(thingy, six.string_types):
         # remove the first XML version/encoding line
         if thingy[0:5].lower() == '<?xml':
-            logger.debug("thingy0: %s" % thingy)
+            logger.debug("thingy0: %s", thingy)
             _part = thingy.split("\n")
             thingy = "".join(_part[1:])
         thingy = thingy.replace(PREFIX, "")
-        logger.debug("thingy: %s" % thingy)
+        logger.debug("thingy: %s", thingy)
         _child = ElementTree.Element('')
         _child.tag = '{%s}FuddleMuddle' % DUMMY_NAMESPACE
         body.append(_child)
         _str = ElementTree.tostring(envelope, encoding="UTF-8")
         if isinstance(_str, six.binary_type):
             _str = _str.decode('utf-8')
-        logger.debug("SOAP precursor: %s" % _str)
+        logger.debug("SOAP precursor: %s", _str)
         # find an remove the namespace definition
         i = _str.find(DUMMY_NAMESPACE)
         j = _str.rfind("xmlns:", 0, i)
@@ -211,14 +257,14 @@ def parse_soap_enveloped_saml(text, body_class, header_class=None):
     :param text: The SOAP object as XML
     :return: header parts and body as saml.samlbase instances
     """
-    envelope = ElementTree.fromstring(text)
+    envelope = defusedxml.ElementTree.fromstring(text)
     assert envelope.tag == '{%s}Envelope' % NAMESPACE
 
-    #print(len(envelope))
+    # print(len(envelope))
     body = None
     header = {}
     for part in envelope:
-        #print(">",part.tag)
+        # print(">",part.tag)
         if part.tag == '{%s}Body' % NAMESPACE:
             for sub in part:
                 try:
@@ -229,11 +275,11 @@ def parse_soap_enveloped_saml(text, body_class, header_class=None):
         elif part.tag == '{%s}Header' % NAMESPACE:
             if not header_class:
                 raise Exception("Header where I didn't expect one")
-            #print("--- HEADER ---")
+            # print("--- HEADER ---")
             for sub in part:
-                #print(">>",sub.tag)
+                # print(">>",sub.tag)
                 for klass in header_class:
-                    #print("?{%s}%s" % (klass.c_namespace,klass.c_tag))
+                    # print("?{%s}%s" % (klass.c_namespace,klass.c_tag))
                     if sub.tag == "{%s}%s" % (klass.c_namespace, klass.c_tag):
                         header[sub.tag] = \
                             saml2.create_class_from_element_tree(klass, sub)
@@ -241,6 +287,7 @@ def parse_soap_enveloped_saml(text, body_class, header_class=None):
 
     return body, header
 
+
 # -----------------------------------------------------------------------------
 
 PACKING = {
