@@ -1,19 +1,24 @@
 #!/usr/bin/env python
 
 import base64
-from saml2.sigver import pre_encryption_part, make_temp, XmlsecError, \
-    SigverError
-from saml2.mdstore import MetadataStore
-from saml2.saml import assertion_from_string, EncryptedAssertion
-from saml2.samlp import response_from_string
-
-from saml2 import sigver, extension_elements_to_elements
+from saml2.xmldsig import SIG_RSA_SHA256
+from saml2 import sigver
+from saml2 import extension_elements_to_elements
 from saml2 import class_name
 from saml2 import time_util
 from saml2 import saml, samlp
 from saml2 import config
+from saml2.sigver import pre_encryption_part
+from saml2.sigver import make_temp
+from saml2.sigver import XmlsecError
+from saml2.sigver import SigverError
+from saml2.mdstore import MetadataStore
+from saml2.saml import assertion_from_string
+from saml2.saml import EncryptedAssertion
+from saml2.samlp import response_from_string
 from saml2.s_utils import factory, do_attribute_statement
 
+import pytest
 from py.test import raises
 
 from pathutils import full_path
@@ -63,7 +68,10 @@ Yj4cAafWaYfjBU2zi1ElwStIaJ5nyp/s/8B8SAPK2T79McMyccP3wSW13LHkmM1j
 wKe3ACFXBvqGQN0IbcH49hu0FKhYFM/GPDJcIHFBsiyMBXChpye9vBaTNEBCtU3K
 jjyG0hRT2mAQ9h+bkPmOvlEo/aH0xR68Z9hw4PF13w=="""
 
-from pyasn1.codec.der import decoder
+try:
+    from pyasn1.codec.der import decoder
+except ImportError:
+    decoder = None
 
 
 def test_cert_from_instance_1():
@@ -76,6 +84,8 @@ def test_cert_from_instance_1():
     assert certs[0] == CERT1
 
 
+@pytest.mark.skipif(not decoder,
+                    reason="pyasn1 is not installed")
 def test_cert_from_instance_ssp():
     xml_response = open(SIMPLE_SAML_PHP_RESPONSE).read()
     response = samlp.response_from_string(xml_response)
@@ -108,6 +118,9 @@ class FakeConfig():
     tmp_key_file = None
     validate_certificate = False
 
+    def getattr(self, attr, default):
+        return getattr(self, attr, default)
+
 
 class TestSecurity():
     def setup_class(self):
@@ -507,9 +520,49 @@ def test_xmlsec_err():
         assert False
 
 
-if __name__ == "__main__":
-    t = TestSecurity()
-    t.setup_class()
-    t.test_verify_1()
+def test_sha256_signing():
+    conf = config.SPConfig()
+    conf.load_file("server_conf")
+    md = MetadataStore([saml, samlp], None, conf)
+    md.load("local", full_path("idp_example.xml"))
 
-    #test_xmlsec_err()
+    conf.metadata = md
+    conf.only_use_keys_in_metadata = False
+    sec = sigver.security_context(conf)
+
+    assertion = factory(
+        saml.Assertion, version="2.0", id="11111",
+        issue_instant="2009-10-30T13:20:28Z",
+        signature=sigver.pre_signature_part("11111", sec.my_cert, 1,
+                                            sign_alg=SIG_RSA_SHA256),
+        attribute_statement=do_attribute_statement(
+            {("", "", "surName"): ("Foo", ""),
+             ("", "", "givenName"): ("Bar", ""), })
+    )
+
+    s = sec.sign_statement(assertion, class_name(assertion),
+                           key_file=full_path("test.key"),
+                           node_id=assertion.id)
+    assert s
+
+
+def test_xmlsec_output_line_parsing():
+    output1 = "prefix\nOK\npostfix"
+    assert sigver.parse_xmlsec_output(output1)
+
+    output2 = "prefix\nFAIL\npostfix"
+    raises(sigver.XmlsecError, sigver.parse_xmlsec_output, output2)
+
+    output3 = "prefix\r\nOK\r\npostfix"
+    assert sigver.parse_xmlsec_output(output3)
+
+    output4 = "prefix\r\nFAIL\r\npostfix"
+    raises(sigver.XmlsecError, sigver.parse_xmlsec_output, output4)
+
+
+if __name__ == "__main__":
+    # t = TestSecurity()
+    # t.setup_class()
+    # t.test_sign_assertion()
+
+    test_sha256_signing()
